/*****************************************************************
 * Fritz Fun                                                     *
 * Created by Jan-Michael Brummer                                *
 * All parts are distributed under the terms of GPL. See COPYING *
 *****************************************************************/

/**
 * \file print.c
 * \brief Print caller list function
 */

#include <ffgtk.h>
#include <cairo-pdf.h>

#define HEADER_HEIGHT	(10*72/25.4)
#define HEADER_GAP		(3*72/25.4)

/** Structure holds information about the page and start positions of columns */
typedef struct {
	gdouble fFontSize;
	GtkTreeView *psView;
	gint nLinesPerPage;
	gint nNumLines;
	gint nNumPages;

	gint nLogoPos;
	gint nDateTimePos;
	gint nNamePos;
	gint nNumberPos;
	gint nLocalNamePos;
	gint nLocalNumberPos;
	gint nDurationPos;
} PrintData;

/**
 * \brief Begin printing, calculate values and store them in PrintData
 * \param psOperation print operation
 * \param psContext print context
 * \param pUserData user data pointer (PrintData)
 */
static void beginPrint( GtkPrintOperation *psOperation, GtkPrintContext *psContext, gpointer pUserData ) {
	PrintData *psData = ( PrintData * ) pUserData;
	GtkTreeModel *psModel;
	GtkTreeIter sIter;
	gboolean bValid;
	double fHeight;

	fHeight = gtk_print_context_get_height( psContext ) - HEADER_HEIGHT - HEADER_GAP;

	psData -> nLinesPerPage = floor( fHeight / psData -> fFontSize ) - 1;
	psData -> nLinesPerPage -= 11;

	Debug( KERN_DEBUG, "fHeight: %f, fFontSize: %f, nLinesPerPage %d\n", fHeight, psData -> fFontSize, psData -> nLinesPerPage );
	psData -> nNumLines = 0;

	psModel = gtk_tree_view_get_model( psData -> psView );
	bValid = gtk_tree_model_get_iter_first( psModel, &sIter );
	while ( bValid ) {
		bValid = gtk_tree_model_iter_next( psModel, &sIter );
		psData -> nNumLines++;
	}

	psData -> nNumPages = ( psData -> nNumLines - 1 ) / psData -> nLinesPerPage + 1;

	psData -> nLogoPos = 5;
	psData -> nDateTimePos = psData -> nLogoPos + 20;
	psData -> nNamePos = psData -> nDateTimePos + 90;
	psData -> nNumberPos = psData -> nNamePos + 130;
	psData -> nLocalNamePos = psData -> nNumberPos + 85;
	psData -> nLocalNumberPos = psData -> nLocalNamePos + 90;
	psData -> nDurationPos = psData -> nLocalNumberPos + 105;

	gtk_print_operation_set_n_pages( psOperation, psData -> nNumPages );
}

/**
 * \brief Short text name
 * \param psCairo cairo pointer
 * \param psLayout pango layout
 * \param pnText text string
 * \param nWidth maximum column width
 */
static void showText( cairo_t *psCairo, PangoLayout *psLayout, gchar *pnText, gint nWidth ) {
	gint nTextWidth, nTextHeight;

	pango_layout_set_text( psLayout, pnText, -1 );
	pango_layout_get_pixel_size( psLayout, &nTextWidth, &nTextHeight );

	if ( nTextWidth > nWidth ) {
		pango_layout_set_width( psLayout, nWidth * PANGO_SCALE );
		pango_layout_set_ellipsize( psLayout, PANGO_ELLIPSIZE_END );
	}
	pango_cairo_show_layout( psCairo, psLayout );
	pango_layout_set_width( psLayout, -1 );
}

/**
 * \brief Draw one single page
 * \param psOperation print operation
 * \param psContext print context
 * \param nPageNr page number to print
 * \param pUserData user data pointer (PrintData)
 */
static void drawPage( GtkPrintOperation *psOperation, GtkPrintContext *psContext, gint nPageNr, gpointer pUserData ) {
	PrintData *psData = ( PrintData * ) pUserData;
	cairo_t *psCairo;
	PangoLayout *psLayout;
	gint nTextWidth, nTextHeight;
	gdouble nWidth;
	gint nLine, nI = 0;
	PangoFontDescription *psDesc;
	gchar *pnPageStr;
	GtkTreeModel *psModel;
	GtkTreeIter sIter;
	GdkPixbuf *psPix, *psDstPix;
	gboolean bValid;
	gchar *pnDateTime;
	gchar *pnName;
	gchar *pnNumber;
	gchar *pnLocalName;
	gchar *pnLocalNumber;
	gchar *pnDuration;
	gint nTmp;
	gint nLineHeight = 0;

	psCairo = gtk_print_context_get_cairo_context( psContext );
	nWidth = gtk_print_context_get_width( psContext );
	cairo_set_source_rgb( psCairo, 0.0, 0.0, 0.0 );

	psLayout = gtk_print_context_create_pango_layout( psContext );
	psDesc = pango_font_description_from_string( "sans 14" );
	pango_layout_set_font_description( psLayout, psDesc );
	pango_font_description_free( psDesc );

	pango_layout_set_text( psLayout, "ffgtk - Anruferliste", -1 );
	pango_layout_get_pixel_size( psLayout, &nTextWidth, &nTextHeight );
	if ( nTextWidth > nWidth ) {
		pango_layout_set_width( psLayout, nWidth );
		pango_layout_set_ellipsize( psLayout, PANGO_ELLIPSIZE_START );
		pango_layout_get_pixel_size( psLayout, &nTextWidth, &nTextHeight );
	}

	cairo_move_to( psCairo, ( nWidth - nTextWidth ) / 2, ( HEADER_HEIGHT - nTextHeight ) / 2 );
	pango_cairo_show_layout( psCairo, psLayout );

	pnPageStr = g_strdup_printf( "%d/%d", nPageNr + 1, psData -> nNumPages );
	pango_layout_set_text( psLayout, pnPageStr, -1 );
	g_free( pnPageStr );

	pango_layout_set_width( psLayout, -1 );
	pango_layout_get_pixel_size( psLayout, &nTextWidth, &nTextHeight );
	cairo_move_to( psCairo, nWidth - nTextWidth - 4, ( HEADER_HEIGHT - nTextHeight ) / 2 );
	pango_cairo_show_layout( psCairo, psLayout );

	g_object_unref( psLayout );

	psLayout = gtk_print_context_create_pango_layout( psContext );
	psDesc = pango_font_description_from_string( "monospace" );
	pango_font_description_set_size( psDesc, psData -> fFontSize * PANGO_SCALE );
	pango_layout_set_font_description( psLayout, psDesc );
	pango_font_description_free( psDesc );

	cairo_move_to( psCairo, 0, HEADER_HEIGHT + HEADER_GAP );
	nLine = nPageNr * psData -> nLinesPerPage;

	psModel = gtk_tree_view_get_model( psData -> psView );
	bValid = gtk_tree_model_get_iter_first( psModel, &sIter );
	nTmp = 0;
	if ( nLine != 0 ) {
		while ( bValid ) {
			bValid = gtk_tree_model_iter_next( psModel, &sIter );
			if ( nTmp++ == nLine ) {
				break;
			}
		}
	}

	nLineHeight = psData -> fFontSize + 2;

	cairo_set_line_width( psCairo, 0 );
	cairo_move_to( psCairo, 0, HEADER_HEIGHT + HEADER_GAP );

	/* Draw header */
	cairo_rectangle( psCairo, 0, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP, nWidth, nLineHeight );
	cairo_set_source_rgb( psCairo, 0.9, 0.9, 0.9 );
	cairo_fill( psCairo );
	cairo_stroke( psCairo );
	cairo_set_source_rgb( psCairo, 0.0, 0.0, 0.0 );

	cairo_move_to( psCairo, psData -> nLogoPos, HEADER_HEIGHT + HEADER_GAP );
	pango_layout_set_text( psLayout, _( "Type" ), -1 );
	pango_cairo_show_layout( psCairo, psLayout );

	cairo_move_to( psCairo, psData -> nDateTimePos, HEADER_HEIGHT + HEADER_GAP );
	pango_layout_set_text( psLayout, _( "Date/Time" ), -1 );
	pango_cairo_show_layout( psCairo, psLayout );

	cairo_move_to( psCairo, psData -> nNamePos, HEADER_HEIGHT + HEADER_GAP );
	pango_layout_set_text( psLayout, _( "Name" ), -1 );
	pango_cairo_show_layout( psCairo, psLayout );

	cairo_move_to( psCairo, psData -> nNumberPos, HEADER_HEIGHT + HEADER_GAP );
	pango_layout_set_text( psLayout, _( "Number" ), -1 );
	pango_cairo_show_layout( psCairo, psLayout );

	cairo_move_to( psCairo, psData -> nLocalNamePos, HEADER_HEIGHT + HEADER_GAP );
	pango_layout_set_text( psLayout, _( "Local Name" ), -1 );
	pango_cairo_show_layout( psCairo, psLayout );

	cairo_move_to( psCairo, psData -> nLocalNumberPos, HEADER_HEIGHT + HEADER_GAP );
	pango_layout_set_text( psLayout, _( "Local Number" ), -1 );
	pango_cairo_show_layout( psCairo, psLayout );

	cairo_move_to( psCairo, psData -> nDurationPos, HEADER_HEIGHT + HEADER_GAP );
	pango_layout_set_text( psLayout, _( "Duration" ), -1 );
	pango_cairo_show_layout( psCairo, psLayout );

	/* print caller rows */
	for ( nI = 1; bValid && nI <= psData -> nLinesPerPage && nLine < psData -> nNumLines; nI++ ) {
		cairo_rectangle( psCairo, 0, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP, nWidth, nLineHeight );
		if ( !( nI & 1 ) ) {
			cairo_set_source_rgb( psCairo, 0.9, 0.9, 0.9 );
			cairo_fill( psCairo );
			cairo_stroke( psCairo );
			cairo_set_source_rgb( psCairo, 0.0, 0.0, 0.0 );
		}
		cairo_stroke( psCairo );

		gtk_tree_model_get( psModel, &sIter, 0, &psPix, 1, &pnDateTime, 2, &pnName, 4, &pnNumber, 5, &pnLocalName, 6, &pnLocalNumber, 7, &pnDuration, -1 );
		psDstPix = gdk_pixbuf_scale_simple( psPix, 10, 10, GDK_INTERP_BILINEAR );

		cairo_save( psCairo );
		gdk_cairo_set_source_pixbuf( psCairo, psDstPix, psData -> nLogoPos, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP + 1 );
		cairo_paint( psCairo );
		cairo_restore( psCairo );
		gdk_pixbuf_unref( psDstPix );

		cairo_move_to( psCairo, psData -> nDateTimePos, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP );
		pango_layout_set_text( psLayout, pnDateTime, -1 );
		pango_cairo_show_layout( psCairo, psLayout );

		cairo_move_to( psCairo, psData -> nNamePos, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP );
		showText( psCairo, psLayout, pnName, psData -> nNumberPos - psData -> nNamePos );

		cairo_move_to( psCairo, psData -> nNumberPos, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP );
		showText( psCairo, psLayout, pnNumber, psData -> nLocalNamePos - psData -> nNumberPos );

		cairo_move_to( psCairo, psData -> nLocalNamePos, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP );
		if ( pnLocalName != NULL && strlen( pnLocalName ) > 0 ) {
			showText( psCairo, psLayout, pnLocalName, psData -> nLocalNumberPos - psData -> nLocalNamePos );
		}

		cairo_move_to( psCairo, psData -> nLocalNumberPos, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP );
		if ( pnLocalNumber != NULL && strlen( pnLocalNumber ) > 0 ) {
			showText( psCairo, psLayout, pnLocalNumber, psData -> nDurationPos - psData -> nLocalNumberPos );
		}

		cairo_move_to( psCairo, psData -> nDurationPos, nI * nLineHeight + HEADER_HEIGHT + HEADER_GAP );
		if ( pnDuration != NULL && strlen( pnDuration ) > 0 ) {
			showText( psCairo, psLayout, pnDuration, nWidth - psData -> nDurationPos );
		}

		bValid = gtk_tree_model_iter_next( psModel, &sIter );
		g_free( pnDateTime );
		g_free( pnName );
		g_free( pnNumber );
		g_free( pnLocalName );
		g_free( pnLocalNumber );
		g_free( pnDuration );

		cairo_rel_move_to( psCairo, 0, nLineHeight );
		nLine++;
	}

	g_object_unref( psLayout );
}

/**
 * \brief Called on end of printing, free PrintData
 * \param psOperation print operation
 * \param psContext print context
 * \param pUserData user data pointer (PrintData)
 */
static void endPrint( GtkPrintOperation *psOperation, GtkPrintContext *psContext, gpointer pUserData ) {
	PrintData *psData = ( PrintData * ) pUserData;

	g_free( psData );
}

/**
 * \brief Print treeview callback function
 * \param psWidget button widget
 * \param pUserData user data pointer (TreeView)
 */
void PrintTreeView( GtkWidget *psWidget, gpointer pUserData ) {
	GtkTreeView *psView = GTK_TREE_VIEW( pUserData );
	GtkPrintOperation *psOperation;
	PrintData *psData;
	GError *psError = NULL;

	psOperation = gtk_print_operation_new();
	psData = g_new0( PrintData, 1 );
	psData -> fFontSize = 10.0;
	psData -> psView = psView;

	g_signal_connect( G_OBJECT( psOperation ), "begin-print", G_CALLBACK( beginPrint ), psData );
	g_signal_connect( G_OBJECT( psOperation ), "draw-page", G_CALLBACK( drawPage ), psData );
	g_signal_connect( G_OBJECT( psOperation ), "end-print", G_CALLBACK( endPrint ), psData );

	gtk_print_operation_run( psOperation, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, NULL, &psError );

	g_object_unref( psOperation );

	if ( psError != NULL ) {
		GtkWidget *psDialog;

		psDialog = gtk_message_dialog_new( NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", psError -> message );
		g_error_free( psError );

		g_signal_connect( psDialog, "response", G_CALLBACK( gtk_widget_destroy ), NULL );
	
		gtk_widget_show( psDialog );
	}
}

/**
 * \brief Create fax report based on give information
 * \param pnFile filename to store report
 * \param pnReportDir save directory
 * \param pnRemote remote number
 * \param pnLocal local number
 * \param pnStatus fax transfer status
 * \param pnError error message
 * \param nPages number of pages
 * \param nBitrate transferred bitrate
 */
void CreateFaxReport( char *pnFile, char *pnReportDir, char *pnRemote, char *pnLocal, char *pnStatus, char *pnError, int nPages, int nBitrate ) {
	cairo_t *psCairo;
	cairo_surface_t *psOut;
	int nWidth, nHeight;
	time_t sTime = time( NULL );
	struct tm *psTime = localtime( &sTime );
	char *pnBuffer;
	GdkPixbuf*psPixBuf;
	struct sPerson *psPerson = NULL;

	if ( pnFile == NULL ) {
		Debug( KERN_WARNING, "File is NULL\n" );
		return;
	}

	if ( pnReportDir == NULL ) {
		Debug( KERN_WARNING, "ReportDir is NULL\n" );
		return;
	}

	psPixBuf = gdk_pixbuf_new_from_file( pnFile, NULL );
	if ( psPixBuf == NULL ) {
		Debug( KERN_WARNING, "PixBuf is null (file '%s')\n", pnFile );
		return;
	}

	nWidth = gdk_pixbuf_get_width( psPixBuf );
	nHeight = gdk_pixbuf_get_height( psPixBuf );
	if ( nWidth != 1728 || nHeight != 2292 ) {
		nWidth = 1728;
		nHeight = 2292;
		GdkPixbuf *psScale = gdk_pixbuf_scale_simple( psPixBuf, nWidth, nHeight, GDK_INTERP_BILINEAR );

		g_object_unref( psPixBuf );
		psPixBuf = psScale;
	}

	pnBuffer = g_strdup_printf( "%s/fax-report_%s_%s-%02d_%02d_%d_%02d_%02d_%02d.pdf",
		pnReportDir, pnLocal, pnRemote,
		psTime -> tm_mday, psTime -> tm_mon + 1, psTime -> tm_year + 1900,
		psTime -> tm_hour, psTime -> tm_min, psTime -> tm_sec );
	psOut = cairo_pdf_surface_create( pnBuffer, nWidth, nHeight + 200 );
	g_free( pnBuffer );

	psCairo = cairo_create( psOut );
	gdk_cairo_set_source_pixbuf( psCairo, psPixBuf, 0, 200 );
	cairo_paint( psCairo );

	cairo_set_source_rgb( psCairo, 0, 0, 0 );
	cairo_select_font_face( psCairo, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD );
	cairo_set_font_size( psCairo, 20.0 );

	cairo_move_to( psCairo, 60.0, 60.0 );
	cairo_show_text( psCairo, "FAX-Report" );

	cairo_select_font_face( psCairo, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL );

	/* Date/Time */
	cairo_move_to( psCairo, 60.0, 95.0 );
	cairo_show_text( psCairo, "Date/Time:" );
	cairo_move_to( psCairo, 220.0, 95.0 );
	pnBuffer = asctime( psTime );
	pnBuffer[ strlen( pnBuffer ) - 1 ] = 0;
	cairo_show_text( psCairo, pnBuffer );

	/* Status */
	cairo_move_to( psCairo, 500.0, 95.0 );
	cairo_show_text( psCairo, "Status:" );
	cairo_move_to( psCairo, 680.0, 95.0 );
	cairo_show_text( psCairo, pnStatus );

	/* Remote name */
	cairo_move_to( psCairo, 60.0, 120.0 );
	cairo_show_text( psCairo, "Remote Name:" );
	psPerson = findPersonByNumber( pnRemote );
	cairo_move_to( psCairo, 220.0, 120.0 );
	cairo_show_text( psCairo, psPerson ? psPerson -> pnDisplayName : _( "Unknown" ) );

	/* Remote number */
	cairo_move_to( psCairo, 500.0, 120.0 );
	cairo_show_text( psCairo, "Remote Number:" );
	cairo_move_to( psCairo, 680.0, 120.0 );
	cairo_show_text( psCairo, pnRemote );

	/* Local name */
	cairo_move_to( psCairo, 60.0, 145.0 );
	cairo_show_text( psCairo, "Local Name:" );
	psPerson = findPersonByNumber( pnLocal );
	cairo_move_to( psCairo, 220.0, 145.0 );
	cairo_show_text( psCairo, psPerson ? psPerson -> pnDisplayName : _( "Unknown" ) );
	
	/* Local number */
	cairo_move_to( psCairo, 500.0, 145.0 );
	cairo_show_text( psCairo, "Local Number:" );
	cairo_move_to( psCairo, 680.0, 145.0 );
	cairo_show_text( psCairo, pnLocal );

	/* Pages */
	cairo_move_to( psCairo, 60.0, 170.0 );
	cairo_show_text( psCairo, "Pages:" );
	cairo_move_to( psCairo, 220.0, 170.0 );
	pnBuffer = g_strdup_printf( "%d", nPages );
	cairo_show_text( psCairo, pnBuffer );
	g_free( pnBuffer );

	/* Bitrate */
	cairo_move_to( psCairo, 500.0, 170.0 );
	cairo_show_text( psCairo, "Bitrate:" );
	cairo_move_to( psCairo, 680.0, 170.0 );
	pnBuffer = g_strdup_printf( "%d", nBitrate );
	cairo_show_text( psCairo, pnBuffer );
	g_free( pnBuffer );

	/* Error message */
	cairo_move_to( psCairo, 60.0, 195.0 );
	cairo_show_text( psCairo, "Message:" );
	cairo_move_to( psCairo, 220.0, 195.0 );
	cairo_show_text( psCairo, pnError );

	/* line */
	cairo_set_line_width( psCairo, 0.5 );
	cairo_move_to( psCairo, 0, 200 );
	cairo_rel_line_to( psCairo, nWidth, 0 );
	cairo_stroke( psCairo );

	cairo_show_page( psCairo );

	cairo_destroy( psCairo );

	cairo_surface_flush( psOut );
	cairo_surface_destroy( psOut );

	g_object_unref( psPixBuf );
}
