/*****************************************************************
 * Fritz Fun                                                     *
 * Created by Jan-Michael Brummer                                *
 * All parts are distributed under the terms of GPL. See COPYING *
 *****************************************************************/

/**
 * \file trayicon.c
 * \brief Trayicon related functions
 */

#include <ffgtk.h>

/** Widget containing trayicon */
static GtkStatusIcon *psTrayIcon = NULL;
/** trayicon open state */
static bool bOpen = FALSE;
/** main gtk loop */
extern GMainLoop *psLoop;
static GtkWidget *psMenu = NULL;

#ifdef HAVE_APPINDICATOR
#include <libappindicator/app-indicator.h>
AppIndicator *psIndicator = NULL;
#endif

/**
 * \brief Set try icon state
 * \param bSet set state
 */
void setTrayOpen( bool bSet ) {
	bOpen = bSet;
}

/**
 * \brief Destroy callback
 * \param psWidget widget
 * \param pData optional data pointer
 */
static void destroy( GtkWidget *psWidget, gpointer pData ) {
	g_main_quit( psLoop );
}

/**
 * \brief Called on tray icon activated, opens or close CallMonitor
 */
static void trayIconActivated( void ) {
	static GtkWidget *psMonitor = NULL;

	if ( bOpen == false ) {
		psMonitor = CallMonitor();
		if ( psMonitor != NULL ) {
			bOpen = TRUE;
		}
	} else {
		if ( psMonitor != NULL ) {
			gtk_widget_hide( GTK_WIDGET( psMonitor ) );
		}
		bOpen = FALSE;
	}
	
#ifndef HAVE_APPINDICATOR
	gtk_status_icon_set_from_pixbuf( GTK_STATUS_ICON( getTrayIcon() ), getTypeIcon( 3 ) );
#else
	app_indicator_set_icon( psIndicator, "callout" );
#endif
}

/**
 * \brief Open addressbook - callback
 */
static void trayIconAddressBook( void ) {
	AddressBook( 0 );
}

/**
 * \brief Select active profile callback
 * \param psItem menu item pressed
 * \param pUserData name of profile
 */
static void trayIconProfile( GtkMenuItem *psItem, gpointer pUserData ) {
	if ( gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM( psItem ) ) == TRUE ) {
		const gchar *pnName = pUserData;
		struct sProfile *psProfile = findProfile( pnName );
		setActiveProfile( psProfile );
		saveProfiles();
	}
}

/**
 * \brief Dial number - callback
 * \param psWidget menu item widget
 * \param pnNumber dial number
 */
static void trayIconDialNumber( GtkWidget *psWidget, gchar *pnNumber ) {
	struct sPerson *psPerson = NULL;

	if ( pnNumber != NULL ) {
		psPerson = findPersonByNumber( pnNumber );
	}

	dialNumberDialog( psPerson, pnNumber );
}

/**
 * \brief Call router reconnect - callback
 */
static void trayIconReconnect( void ) {
	routerReconnect( getActiveProfile() );
}

/**
 * \brief Copy external ip to clipboard - callback
 */
static void trayIconIpToClipboard( void ) {
	gchar *pnIp = routerGetIp( getActiveProfile() );

	if ( pnIp != NULL ) {
		gtk_clipboard_set_text( gtk_clipboard_get( GDK_NONE ), pnIp, -1 );
		g_free( pnIp );
	}
}

/**
 * \brief Mute sounds - callback
 * \param psItem menu item widget
 * \param pUserData UNUSED
 */
static void trayIconMute( GtkCheckMenuItem *psItem, gpointer pUserData ) {
	gboolean bMute = callMonitorGetMute( getActiveProfile() );

	gtk_check_menu_item_set_active( psItem, !bMute );
	callMonitorSetMute( getActiveProfile(), !bMute );
}

/**
 * \brief Add last call to submenu - callback
 * \param psMenu connect each entry to this menu
 */
void addLastcalls( GtkWidget *psMenu ) {
	GList *psListIn = NULL;
	GList *psListOut = NULL;
	GList *psListMissed = NULL;
	GtkWidget *psItem = NULL;
	GtkWidget *psImage = NULL;
	GdkPixbuf *psPix = NULL;
	GtkWidget *psSubMenu = NULL;

	psItem = gtk_image_menu_item_new_with_label( _( "Last calls" ) );
	psSubMenu = gtk_menu_new();
	GtkWidget *psLastCalls = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psLastCalls );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psItem ), psSubMenu );
	gtk_menu_append( psMenu, psItem );

	psListIn = getLastcalls( 0 );
	psListOut = getLastcalls( 1 );
	psListMissed = getLastcalls( 2 );

	if ( psListIn == NULL && psListOut == NULL && psListMissed == NULL ) {
		gtk_widget_set_sensitive( GTK_WIDGET( psItem ), FALSE );
		return;
	}

	if ( psListIn != NULL ) {
#ifdef HAVE_APPINDICATOR
		psItem = gtk_menu_item_new_with_label( _( "Incoming" ) );
		gtk_menu_append( psSubMenu, psItem );
#endif

		psItem = gtk_separator_menu_item_new();
		gtk_menu_append( psSubMenu, psItem );

		while ( psListIn != NULL ) {
			psItem = gtk_image_menu_item_new_with_label( psListIn -> data );
			psPix = getTypeIcon( 1 );
			psImage = GTK_WIDGET( gtk_image_new_from_pixbuf( psPix ) );
			gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psImage );
#if GTK_MINOR_VERSION >= 16
            gtk_image_menu_item_set_always_show_image( GTK_IMAGE_MENU_ITEM( psItem ), TRUE );
#endif
			gtk_menu_append( psSubMenu, psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), psListIn -> data );
			gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );
			psListIn = psListIn -> next;
		}
	}

	if ( psListOut != NULL ) {
#ifdef HAVE_APPINDICATOR
		psItem = gtk_menu_item_new_with_label( _( "Outgoing" ) );
		gtk_menu_append( psSubMenu, psItem );
#endif

		psItem = gtk_separator_menu_item_new();
		gtk_menu_append( psSubMenu, psItem );

		while ( psListOut != NULL ) {
			psItem = gtk_image_menu_item_new_with_label( psListOut -> data );
			psPix = getTypeIcon( 3 );
			psImage = GTK_WIDGET( gtk_image_new_from_pixbuf( psPix ) );
			gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psImage );
#if GTK_MINOR_VERSION >= 16
            gtk_image_menu_item_set_always_show_image( GTK_IMAGE_MENU_ITEM( psItem ), TRUE );
#endif
			gtk_menu_append( psSubMenu, psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), psListOut -> data );
			gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );
			psListOut = psListOut -> next;
		}
	}

	if ( psListMissed != NULL ) {
#ifdef HAVE_APPINDICATOR
		psItem = gtk_menu_item_new_with_label( _( "Missed" ) );
		gtk_menu_append( psSubMenu, psItem );
#endif

		psItem = gtk_separator_menu_item_new();
		gtk_menu_append( psSubMenu, psItem );

		while ( psListMissed != NULL ) {
			psItem = gtk_image_menu_item_new_with_label( psListMissed -> data );
			psPix = getTypeIcon( 2 );
			psImage = GTK_WIDGET( gtk_image_new_from_pixbuf( psPix ) );
			gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psImage );
#if GTK_MINOR_VERSION >= 16
            gtk_image_menu_item_set_always_show_image( GTK_IMAGE_MENU_ITEM( psItem ), TRUE );
#endif
			gtk_menu_append( psSubMenu, psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), psListMissed -> data );
			gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );
			psListMissed = psListMissed -> next;
		}
	}
}

/**
 * \brief Set diversity state - callback
 * \param psItem menu item widget
 * \param psDiversity diversity structure
 */
static void trayIconSetDiversity( GtkMenuItem *psItem, struct sDiversity *psDiversity ) {
	psDiversity -> bSet = !psDiversity -> bSet;

	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), psDiversity -> bSet );
	routerSetDiversity( getActiveProfile(), psDiversity -> nIndex, psDiversity -> bSet );
}

/**
 * \brief Send SMS - callback
 * \param psItem menu item widget
 * \param pUserData UNUSED
 */
static void trayIconSendSms( GtkMenuItem *psItem, gpointer pUserData ) {
	Debug( KERN_INFO, "Send SMS\n" );
	sendSmsDialog();
}

/**
 * \brief Open help - callback
 * \param psItem menu item widget
 * \param psDiversity diversity structure
 */
static void trayIconHelp( GtkMenuItem *psItem, struct sDiversity *psDiversity ) {
	gchar *pnExec = NULL;

	/* create execution command line for g_spawn */
	pnExec = g_strdup( "xdg-open http://www.tabos.org/forum" );
	//pnExec = g_strdup( "xdg-open http://wiki.ubuntuusers.de/ffgtk" );
	//pnExec = g_strdup_printf( "xdg-open %s/index.html", HTMLDIR );
	
	/* execute xdg-open, the standard ui-independent way in opening the default browser */
	g_spawn_command_line_async( pnExec, NULL );

	/* free command line */
	g_free( pnExec );
}

/**
 * \brief Add diversity information to submenu
 * \param psSubMenu add all information to this submenu
 */
static void addDiversity( GtkWidget *psSubMenu ) {
	struct sDiversity *psDiversity = NULL;
	GList *psList = NULL;
	GtkWidget *psItem = NULL;

	psItem = gtk_image_menu_item_new_with_label( _( "Diversity" ) );
	GtkWidget *psSubMenu3 = gtk_menu_new();
	GtkWidget *psDiversityImage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psDiversityImage );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psItem ), psSubMenu3 );
	gtk_menu_append( psSubMenu, psItem );

	psList = routerGetDiversity();
	if ( psList == NULL || g_list_length( psList ) == 0 ) {
		gtk_widget_set_sensitive( psItem, FALSE );
		return;
	}

	for ( psList = routerGetDiversity(); psList != NULL && psList -> data != NULL; psList = psList -> next ) {
		psDiversity = psList -> data;

		psItem = gtk_check_menu_item_new_with_label( psDiversity -> pnName );
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), psDiversity -> bSet );
		gtk_menu_append( psSubMenu3, psItem );
		g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconSetDiversity ), psDiversity );
	}
}

static GtkWidget *createMenu( void ) {
	GtkWidget *psItem;
	GtkWidget *psMenu;

	psMenu = gtk_menu_new();

#if defined HAVE_APPINDICATOR || defined __MACOSX__
	/* Callmonitor */
	psItem = gtk_image_menu_item_new_with_label( _( "Call Monitor" ) );
	GtkWidget *psMonitor = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psMonitor );
	gtk_menu_append( psMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconActivated ), NULL );
#endif
	
	/* Addressbook */
	psItem = gtk_image_menu_item_new_with_label( _( "Address Book" ) );
	GtkWidget *psAddress = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psAddress );
	gtk_menu_append( psMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconAddressBook ), NULL );

	/* Last calls */
	addLastcalls( psMenu );

	/* Functions */
	psItem = gtk_image_menu_item_new_with_label( _( "Functions" ) );
	GtkWidget *psSubMenu1 = gtk_menu_new();
	GtkWidget *psFunctions = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psFunctions );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psItem ), psSubMenu1 );
	gtk_menu_append( psMenu, psItem );

	/* Functions - Call number */
	psItem = gtk_image_menu_item_new_with_label( _( "Call number" ) );
	GtkWidget *psCallMonitor = GTK_WIDGET( gtk_image_new_from_pixbuf( getTypeIcon( 3 ) ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psCallMonitor );
	gtk_menu_append( psSubMenu1, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), NULL );
	gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );

	/* Functions - Reconnect */
	psItem = gtk_image_menu_item_new_with_label( _( "Reconnect" ) );
	GtkWidget *psReconnect = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psReconnect );
	gtk_menu_append( psSubMenu1, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconReconnect ), NULL );

	/* Functions - Copy IP */
	psItem = gtk_image_menu_item_new_with_label( _( "Copy IP" ) );
	GtkWidget *psClipboard = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_COPY, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psClipboard );
	gtk_menu_append( psSubMenu1, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconIpToClipboard ), NULL );

	/* Functions - Send SMS */
	psItem = gtk_image_menu_item_new_with_label( _( "Send SMS" ) );
	GtkWidget *psSendSms = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psSendSms );
	gtk_menu_append( psSubMenu1, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconSendSms ), NULL );
	gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );

	/* Functions - Diversity */
	addDiversity( psSubMenu1 );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_append( psMenu, psItem );

	/* Preferences */
	psItem = gtk_image_menu_item_new_with_label( _( "Preferences" ) );
	GtkWidget *psPref = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psPref );
	gtk_menu_append( psMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( preferences ), NULL );

	/* Profiles */
	psItem = gtk_image_menu_item_new_with_label( _( "Profiles" ) );
	GtkWidget *psSubMenu = gtk_menu_new();
	GtkWidget *psCall = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psCall );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psItem ), psSubMenu );
	gtk_menu_append( psMenu, psItem );

	/* Profiles - Manage */
	psItem = gtk_image_menu_item_new_with_label( _( "Manage" ) );
	GtkWidget *psManage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psManage );
	gtk_menu_append( psSubMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( ProfilesManage ), NULL );

	/* Profiles - Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_append( psSubMenu, psItem );

	/* Profiles - Profile list */
	GList *psList = getProfiles();
	struct sProfile *psProfile = NULL;
	psItem = NULL;
	GSList *psGroup = NULL;

	while ( psList != NULL ) {
		psProfile = psList -> data;
		if ( psProfile != NULL ) {
			psItem = gtk_radio_menu_item_new_with_label( psGroup, psProfile -> pnName );
			psGroup = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM( psItem ) );
			if ( psProfile == getActiveProfile() ) {
				gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), TRUE );
			}
			gtk_menu_append( psSubMenu, psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconProfile ), psProfile -> pnName );
		}
		psList = psList -> next;
	}

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_append( psMenu, psItem );

	/* Mute */
	psItem = gtk_check_menu_item_new_with_label( _( "Mute" ) );
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), callMonitorGetMute( getActiveProfile() ) );
	gtk_menu_append( psMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconMute ), NULL );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_append( psMenu, psItem );

	/* Help */
	//psItem = gtk_image_menu_item_new_with_label( _( "Online-Help" ) );
	psItem = gtk_image_menu_item_new_with_label( _( "Help" ) );
	GtkWidget *psHelp = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_HELP, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psHelp );
	gtk_menu_append( psMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconHelp ), NULL );

	/* About */
	psItem = gtk_image_menu_item_new_with_label( _( "About" ) );
	GtkWidget *psAbout = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psAbout );
	gtk_menu_append( psMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( aboutDialog ), NULL );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_append( psMenu, psItem );

	/* Quit */
	psItem = gtk_image_menu_item_new_with_label( _( "Quit" ) );
	GtkWidget *psQuit = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_QUIT, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psQuit );
	gtk_menu_append( psMenu, psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( destroy ), NULL );

	return psMenu;
}

/**
 * \brief Handle tray icon menu
 * \param psStatusIcon status icon pointer
 * \param nButton which button
 * \param nActivationTime activation time
 * \param pUserData user data pointer
 */
static void trayIconOnMenu( GtkStatusIcon *psStatusIcon, guint nButton, guint nActivateTime, gpointer pUserData) {
	if ( psMenu == NULL ) {
		psMenu = createMenu();
	}

	gtk_widget_show_all( psMenu );

	gtk_menu_popup( GTK_MENU( psMenu ), NULL, NULL, gtk_status_icon_position_menu, psStatusIcon, nButton, nActivateTime );
}

/**
 * \brief Create tray icon and set menu/tooltip
 */
void createTrayIcon( void ) {
	psTrayIcon = gtk_status_icon_new();
	g_signal_connect( G_OBJECT( psTrayIcon ), "popup-menu", G_CALLBACK( trayIconOnMenu ), NULL );
	g_signal_connect( G_OBJECT( psTrayIcon ), "activate", G_CALLBACK( trayIconActivated ), NULL );
	gtk_status_icon_set_from_pixbuf( psTrayIcon, getTypeIcon( 3 ) );
	gtk_status_icon_set_tooltip( psTrayIcon,  "ffgtk" );
	gtk_status_icon_set_visible( psTrayIcon, TRUE );
}


/**
 * \brief Periodically update tooltip information
 * \param pUserData pointer to tray icon
 * \return error code
 */
gpointer updateTooltip( gpointer pUserData ) {
	gchar *pnIp = NULL;
	gchar *pnText = NULL;
	gint nMaxUp = 0;
	gint nMaxDown = 0;

	while ( 1 ) {
		pnIp = routerGetIp( getActiveProfile() );

		if ( pnIp != NULL ) {
			routerGetSpeed( getActiveProfile(), &nMaxUp, &nMaxDown );
			pnText = g_strdup_printf( _( "External IP: %s\nMax. Up: %d kbit/s\nMax. Down: %d kbit/s" ), pnIp, nMaxUp >> 10, nMaxDown >> 10 );
			g_free( pnIp );
			pnIp = NULL;
		} else {
			pnText = g_strdup_printf( _( "Not connected" ) );
		}

#ifndef HAVE_APPINDICATOR
		GtkStatusIcon *psIcon = ( GtkStatusIcon * ) pUserData;
		gdk_threads_enter();
		gtk_status_icon_set_tooltip( psIcon, pnText );
		gdk_threads_leave();
#endif

		g_free( pnText );
		pnText = NULL;

		sleep( 15 );
	}

	return NULL;
}

/**
 * \brief Get tray icon widget
 * \return trayicon widget
 */
GtkStatusIcon *getTrayIcon( void ) {
	return psTrayIcon;
}

void recreateAppMenu( void ) {
#ifdef HAVE_APPINDICATOR
	if ( psMenu != NULL ) {
		gtk_widget_destroy( psMenu );
	}
	psMenu = createMenu();
	gtk_widget_show_all( GTK_WIDGET( psMenu ) );
	app_indicator_set_menu( psIndicator, GTK_MENU( psMenu ) );
#endif
}

/**
 * \brief Initialize tray icon structure and start update tooltip thread
 */
void TrayIconInit( void ) {
#ifdef HAVE_APPINDICATOR
	psIndicator = app_indicator_new_with_path( "ffgtk", "callout", APP_INDICATOR_CATEGORY_APPLICATION_STATUS, SHAREDIR );
	Debug( KERN_DEBUG, "psIndicator %p\n", psIndicator );
	app_indicator_set_status( psIndicator, APP_INDICATOR_STATUS_ACTIVE );

	recreateAppMenu();
#else
	createTrayIcon();
	g_thread_create( updateTooltip, psTrayIcon, false, NULL );
#endif

}
