/*****************************************************************
 * Fritz Fun                                                     *
 * Created by Jan-Michael Brummer                                *
 * All parts are distributed under the terms of GPL. See COPYING *
 *****************************************************************/

/**
 * \file audio_ao.c
 * \brief libao audio plugin
 */

#include <ffgtk.h>
#include <preferences.h>
#include <ao/ao.h>

/** predefined backup values */
static gint nAoChannels = 2;
static gint nAoSampleRate = 8000;
static gint nAoBitsPerSample = 16;
static int nDefaultDriver = -1;

/**
 * \brief Detect supported audio devices
 * \return 0
 */
static int aoAudioDetectDevices( void ) {
	ao_initialize();

	nDefaultDriver = ao_default_driver_id();

	return nDefaultDriver != 0;
}

/**
 * \brief Initialize audio device
 * \param nChannels number of channels
 * \param nSampleRate sample rate
 * \param nBitsPerSample number of bits per samplerate
 * \return TRUE on success, otherwise error
 */
static int aoAudioInit( unsigned char nChannels, unsigned short nSampleRate, unsigned char nBitsPerSample ) {
	/* TODO: Check if configuration is valid and usable */
	nAoChannels = nChannels;
	nAoSampleRate = nSampleRate;
	nAoBitsPerSample = nBitsPerSample;

	return 0;
}

/**
 * \brief Open audio device
 * \return private data or NULL on error
 */
static void *aoAudioOpen( void ) {
	ao_device *psDevice = NULL;
	ao_sample_format sFormat;

	memset( &sFormat, 0, sizeof( sFormat ) );

	sFormat.bits = nAoBitsPerSample;
	sFormat.channels = nAoChannels;
	sFormat.rate = nAoSampleRate;
	sFormat.byte_format = AO_FMT_LITTLE;

	psDevice = ao_open_live( nDefaultDriver, &sFormat, NULL );

	return psDevice;
}

/**
 * \brief Write audio data
 * \param pPriv private data
 * \param pnData audio data
 * \param nSize size of audio data
 * \return bytes written or error code
 */
static int aoAudioWrite( void *pPriv, unsigned char *pnData, unsigned int nSize ) {
	ao_device *psDevice = pPriv;

	ao_play( psDevice, ( char * ) pnData, nSize );

	return nSize;
}

/**
 * \brief Stop and remove pipeline
 * \param pPriv private data
 * \return error code
 */
int aoAudioClose( void *pPriv ) {
	ao_device *psDevice = pPriv;

	if ( psDevice != NULL ) {
		ao_close( psDevice );
	}

	return 0;
}

int aoAudioDeinit( void ) {
	return 0;
}

/** audio definition */
struct sAudio sAo = {
	aoAudioInit,
	aoAudioOpen,
	aoAudioWrite,
	NULL,
	aoAudioClose,
	aoAudioDeinit,
	NULL
};

MODULE_INIT( PLUGIN_TYPE_AUDIO, _( "AO" ), &sAo, NULL, aoAudioDetectDevices );
