/*****************************************************************
 * Fritz Fun                                                     *
 * Created by Jan-Michael Brummer                                *
 * All parts are distributed under the terms of GPL. See COPYING *
 *****************************************************************/

/**
 * \file audio_gstreamer.c
 * \brief GStreamer audio plugin
 */

#include <ffgtk.h>
#include <preferences.h>
#include <gst/interfaces/propertyprobe.h>
#include <gst/app/gstappsrc.h>
#include <gst/app/gstappbuffer.h>

/** Internal device list */
static GList *psDeviceList = NULL;
/** This flag informs about gstreamer initialization status */
static gboolean bAudioInitFailed = TRUE;
/** predefined backup values */
static gint nGstreamerChannels = 2;
static gint nGstreamerSampleRate = 8000;
static gint nGstreamerBitsPerSample = 16;

/** Device information, description and name */
struct sDeviceInfo {
	gchar *pnDescr;
	gchar *pnName;
};

/** Device test list */
struct sDeviceTest {
	gchar *pnSink;
	gchar *pnTest;
} asDeviceTest[] = {
	{ "pulsesink", "pulsesinkpresencetest" },
	{ "alsasink", "alsasinkpresencetest" },
	{ NULL, NULL }
};

/** Get and set selected gstreamer device */
CREATE_STRING_PREFS( gstreamer, SelectedDevice, "/plugins/gstreamer/device" );

/**
 * \brief Stop and clean gstreamer pipeline
 * \param psBus gstreamer bus
 * \param psMessage gstreamer message
 * \param pPipeline pipeline to pipeline we want to stop
 * \return TRUE if we received a EOS for the requested pipeline, otherwise FALSE
 */
static gboolean pipelineCleaner( GstBus *psBus, GstMessage *psMessage, gpointer pPipeline ) {
	gboolean bResult = TRUE;

	/* If we receive a End-Of-Stream message for the requested pipeline, stop the pipeline */
	if ( GST_MESSAGE_TYPE( psMessage ) == GST_MESSAGE_EOS && GST_MESSAGE_SRC( psMessage ) == GST_OBJECT_CAST( pPipeline ) ) {
		bResult = FALSE;
		gst_element_set_state( GST_ELEMENT( pPipeline ), GST_STATE_NULL );
		g_object_unref( pPipeline );
	}

	return bResult;
}

/**
 * \brief Detect supported audio devices
 * \return 0
 */
static int gstreamerAudioDetectDevices( void ) {
	GError *psError = NULL;
	int nDevice = 0;

	/* initialize gstreamer */
	bAudioInitFailed = !gst_init_check( NULL, NULL, &psError );
	if ( bAudioInitFailed == TRUE ) {
		Debug( KERN_WARNING, _( "GStreamer failed to initialized (%s)\n" ), psError ? psError -> message : "" );
		if ( psError != NULL ) {
			g_error_free( psError );
			psError = NULL;
		}
		return -1;
	}

	while ( asDeviceTest[ nDevice ].pnSink != NULL ) {
		GstElement *psElement = NULL;
		guint nIndex = 0;

		/* check if we have a alsa sink within gstreamer */
		psElement = gst_element_factory_make( asDeviceTest[ nDevice ].pnSink, asDeviceTest[ nDevice ].pnTest );
		if ( psElement != NULL ) {
			GstPropertyProbe *psProbe = NULL;
			const GParamSpec *psSpec = NULL;
			GValueArray *psArray = NULL;

			/* Set playback mode to pause */
			gst_element_set_state( psElement, GST_STATE_PAUSED );
			psProbe = GST_PROPERTY_PROBE( psElement );
			psSpec = gst_property_probe_get_property( psProbe, "device" );

			/* now try to receive the device name and add it to the internal list */
			psArray = gst_property_probe_probe_and_get_values( psProbe, psSpec );
			if ( psArray != NULL ) {
				for ( nIndex = 0; nIndex < psArray -> n_values; nIndex++ ) {
					GValue *psDevice = NULL;
					gchar *pnName = NULL;

					psDevice = g_value_array_get_nth( psArray, nIndex );
					g_object_set_property( G_OBJECT( psElement ), "device", psDevice );
					g_object_get( G_OBJECT( psElement ), "device-name", &pnName, NULL );
					if ( pnName == NULL ) {
						g_object_get( G_OBJECT( psElement ), "device", &pnName, NULL );
					}
					if ( pnName != NULL ) {
						struct sDeviceInfo *psInfo = g_malloc( sizeof( *psInfo ) );
						psInfo -> pnDescr = g_strdup_printf( "%s device=%s", asDeviceTest[ nDevice ].pnSink, g_value_get_string( psDevice ) );
						if ( nDevice == 0 ) {
							Debug( KERN_DEBUG, "pnName: %s\n", pnName );
							gchar *pnSpec = strrchr( pnName, '.' );
							if ( pnSpec != NULL ) {
								psInfo -> pnName = g_strdup_printf( "Pulseaudio (%s)", pnSpec + 1 );
							} else {
								psInfo -> pnName = g_strdup( "Pulseaudio" );
							}
						} else {
							psInfo -> pnName = g_strdup( pnName );
						}
						Debug( KERN_DEBUG, "Sink detected: %s (%s)\n", psInfo -> pnDescr, psInfo -> pnName );
						psDeviceList = g_list_append( psDeviceList, psInfo );
						g_free( pnName );
					}
				}
				g_value_array_free( psArray );
			}

			gst_element_set_state( psElement, GST_STATE_NULL );
			gst_object_unref( GST_OBJECT( psElement ) );
		}

		nDevice++;
	}

	return 0;
}

/**
 * \brief Set buffer size we want to use
 * \param nBufferSize requested buffer size
 */
static void gstreamerAudioSetBufferSize( gpointer pPriv, unsigned nBufferSize ) {
	GstElement *psSrc = NULL;
	GstElement *psPipeline = pPriv;

	/* Try to receive pipeline bin name */
	psSrc = gst_bin_get_by_name( GST_BIN( psPipeline ), "ffgtk_src" );
	if ( psSrc != NULL ) {
		/* set blocksize */
		g_object_set( G_OBJECT( psSrc ), "blocksize", nBufferSize, NULL );
		g_object_unref( psSrc );
	}
}

/**
 * \brief Initialize audio device
 * \param nChannels number of channels
 * \param nSampleRate sample rate
 * \param nBitsPerSample number of bits per samplerate
 * \return TRUE on success, otherwise error
 */
static int gstreamerAudioInit( unsigned char nChannels, unsigned short nSampleRate, unsigned char nBitsPerSample ) {
	/* TODO: Check if configuration is valid and usable */
	nGstreamerChannels = nChannels;
	nGstreamerSampleRate = nSampleRate;
	nGstreamerBitsPerSample = nBitsPerSample;

	return 0;
}

/**
 * \brief Open audio device
 * \return private data or NULL on error
 */
static void *gstreamerAudioOpen( void ) {
	gchar *pnCommand = NULL;
	GError *psError = NULL;
	GstState sCurrent;
	GstElement *psPipeline = NULL;

	Debug( KERN_DEBUG, "started\n" );
	if ( gstreamerGetSelectedDevice( getActiveProfile() ) == NULL ) {
		return NULL;
	}

	/* Create command for pipeline */
	pnCommand = g_strdup_printf( "appsrc is-live=true name=ffgtk_src"
		" ! audio/x-raw-int"
		",rate=%d"
		",channels=%d"
		",width=%d"
		",depth=%d"
		",signed=true,endianness=1234"
		" ! %s",
		nGstreamerSampleRate, nGstreamerChannels, nGstreamerBitsPerSample, nGstreamerBitsPerSample,
		gstreamerGetSelectedDevice( getActiveProfile() ) );
	Debug( KERN_DEBUG, "pnCommand %s\n", pnCommand );
	
	/* Start pipeline */
	psPipeline = gst_parse_launch( pnCommand, &psError );
	if ( psError == NULL ) {
		/* Try to set pipeline state to playing */
		gst_element_set_state( psPipeline, GST_STATE_PLAYING );

		/* Now read the current state and verify that we have a good pipeline */
		gst_element_get_state( psPipeline, &sCurrent, NULL, GST_SECOND );

		if ( !( sCurrent == GST_STATE_PLAYING || sCurrent == GST_STATE_PAUSED ) ) {
			gst_element_set_state( psPipeline, GST_STATE_NULL );
			gst_object_unref( GST_OBJECT( psPipeline ) );
			psPipeline = NULL;
		} else {
			Debug( KERN_DEBUG, "Ok\n" );
		}

		gstreamerAudioSetBufferSize( psPipeline, 160 );
	} else {
		g_error_free( psError );
		psPipeline = NULL;
	}

	g_free( pnCommand );

	return psPipeline;
}

/**
 * \brief Write audio data
 * \param pPriv private data
 * \param pnData audio data
 * \param nSize size of audio data
 * \return bytes written or error code
 */
static int gstreamerAudioWrite( void *pPriv, unsigned char *pnData, unsigned int nSize ) {
	gchar *pnTmp = NULL;
	GstBuffer *psBuffer = NULL;
	GstElement *psSrc = NULL;
	GstElement *psPipeline = pPriv;
	unsigned int nWritten = -1;

	g_return_val_if_fail( GST_IS_BIN( psPipeline ), -2 );

	psSrc = gst_bin_get_by_name( GST_BIN( psPipeline ), "ffgtk_src" );
	if ( psSrc != NULL ) {
		pnTmp = ( gchar * ) g_malloc0( nSize );
		memcpy( ( char * ) pnTmp, ( char * ) pnData, nSize );
		psBuffer = gst_app_buffer_new( pnTmp, nSize, ( GstAppBufferFinalizeFunc ) g_free, pnTmp );
		gst_app_src_push_buffer( GST_APP_SRC( psSrc ), psBuffer );
		nWritten = nSize;
		g_object_unref( psSrc );
	}

	return nWritten;
}

/**
 * \brief Stop and remove pipeline
 * \param pPriv private data
 * \return error code
 */
int gstreamerAudioClose( void *pPriv ) {
	GstElement *psPipeline = pPriv;

	if ( psPipeline != NULL ) {
		GstElement *psSrc = gst_bin_get_by_name( GST_BIN( psPipeline ), "ffgtk_src" );
		if ( psSrc != NULL ) {
			GstBus *psBus = gst_pipeline_get_bus( GST_PIPELINE( psPipeline ) );
			gst_bus_add_watch( psBus, pipelineCleaner, psPipeline );
			gst_app_src_end_of_stream( GST_APP_SRC( psSrc ) );
			gst_object_unref( psBus );
			psPipeline = NULL;
		}
	}

	return 0;
}

int gstreamerAudioDeinit( void ) {
	return 0;
}

/**
 * \brief Display gstreamer preferences window
 */
static void gstreamerPreferences( void ) {
	GtkWidget *psDialog;
	GtkWidget *psComboBox;
	GtkWidget *psLabel;
	int nIndex = 0;
	GList *psList = psDeviceList;
	struct sProfile *psProfile = getActiveProfile();

	psDialog = gtk_dialog_new_with_buttons( _( "GStreamer Preferences" ), NULL, 0, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL );
	psComboBox = gtk_combo_box_new_text();
	psLabel = gtk_label_new( "" );

	gtk_label_set_markup( GTK_LABEL( psLabel ), _( "<b>Select sound device:</b>" ) );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( psDialog ) -> vbox ), psLabel, FALSE, TRUE, 10 );

	while ( psList != NULL && psList -> data != NULL ) {
		struct sDeviceInfo *psInfo = psList -> data;

		gtk_combo_box_append_text( GTK_COMBO_BOX( psComboBox ), psInfo -> pnName );

		if ( gstreamerGetSelectedDevice( psProfile ) != NULL ) {
			if ( !strcmp( psInfo -> pnDescr, gstreamerGetSelectedDevice( psProfile ) ) ) {
				gtk_combo_box_set_active( GTK_COMBO_BOX( psComboBox ), nIndex );
			}
		} else {
			gtk_combo_box_set_active( GTK_COMBO_BOX( psComboBox ), 0 );
		}
		nIndex++;

		psList = psList -> next;
	}
	
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( psDialog ) -> vbox ), psComboBox, FALSE, TRUE, 5 );
	gtk_widget_set_size_request( psDialog, 300, 120 );
	gtk_widget_show( GTK_WIDGET( psLabel ) );
	gtk_widget_show( GTK_WIDGET( psComboBox ) );
	gtk_dialog_run( GTK_DIALOG( psDialog ) );

	prefsAddNone( getActiveProfile(), "/plugins/gstreamer" );

	psList = psDeviceList;
	nIndex = 0;

	while ( psList != NULL && psList -> data != NULL ) {
		struct sDeviceInfo *psInfo = psList -> data;

		if ( gtk_combo_box_get_active( GTK_COMBO_BOX( psComboBox ) ) == nIndex ) {
			gstreamerSetSelectedDevice( psProfile, psInfo -> pnDescr );
			break;
		}

		psList = psList -> next;
		nIndex++;
	}

	gtk_widget_destroy( psDialog );
	SavePreferences( getActiveProfile() );

	// TODO: Reload driver
}

/** audio definition */
struct sAudio sGstreamer = {
	gstreamerAudioInit,
	gstreamerAudioOpen,
	gstreamerAudioWrite,
	NULL,
	gstreamerAudioClose,
	gstreamerAudioDeinit,
	gstreamerPreferences
};

MODULE_INIT( PLUGIN_TYPE_AUDIO, _( "GStreamer" ), &sGstreamer, NULL, gstreamerAudioDetectDevices );
