/*****************************************************************
 * Fritz Fun                                                     *
 * Created by Jan-Michael Brummer                                *
 * All parts are distributed under the terms of GPL. See COPYING *
 *****************************************************************/

/**
 * \file fax_capifax.c
 * \brief Capifax plugin
 */

#include <libcapifax.h>
#include <ffgtk.h>

static GtkWidget *psStatusLabel, *psPageNumber, *psBadRowsNumber, *psErrorLabel,
	*psEncodingNumber, *psBitrateNumber, *psOkButton, *psCancelButton, *psProgressBar;
static char *pnCurrentFile = NULL;
static int nCurrentBitrate = 0;
static int nCurrentEcm = 0;
static int nCurrentController = 0;
static gchar *pnCurrentNumber = NULL;
static gchar *pnCurrentSenderMsn = NULL;
static gchar *pnCurrentSenderNumber = NULL;
static gchar *pnCurrentSenderName = NULL;
static int nCurrentCallAnonymous = 0;
static int nCurrentSize = 0;
static gboolean bSending = false;
static int nCapiFaxInit = -1;
static int nTimer = 0;
static char *pnCurrentReportDir = NULL;
static GtkWidget *psSendWindow = NULL;

/**
 * \brief Convert error id into string (TODO: set routine with translation to libcapi?)
 * \param nReason error id
 * \return error in string
 */
static char *capiInfo2String( int nReason ) {
	switch ( nReason ) {
		/*-- informative values (corresponding message was processed) -----*/
		case 0x0001:
			return _( "NCPI not supported by current protocol, NCPI ignored" );
		case 0x0002:
			return _( "Flags not supported by current protocol, flags ignored" );
		case 0x0003:
		   return _( "Alert already sent by another application" );

		/*-- error information concerning CAPI_REGISTER -----*/
		case 0x1001:
		   return _( "Too many applications" );
		case 0x1002:
		   return _( "Logical block size too small, must be at least 128 Bytes" );
		case 0x1003:
		   return _( "Buffer exceeds 64 kByte" );
		case 0x1004:
		   return _( "Message buffer size too small, must be at least 1024 Bytes" );
		case 0x1005:
		   return _( "Max. number of logical connections not supported" );
		case 0x1006:
		   return _( "Reserved" );
		case 0x1007:
		   return _( "The message could not be accepted because of an internal busy condition" );
		case 0x1008:
		   return _( "OS resource error (no memory ?)" );
		case 0x1009:
		   return _( "CAPI not installed" );
		case 0x100A:
		   return _( "Controller does not support external equipment" );
		case 0x100B:
		   return _( "Controller does only support external equipment" );

		/*-- error information concerning message exchange functions -----*/
		case 0x1101:
		   return _( "Illegal application number" );
		case 0x1102:
		   return _( "Illegal command or subcommand or message length less than 12 bytes" );
		case 0x1103:
		   return _( "The message could not be accepted because of a queue full condition !! The error code does not imply that CAPI cannot receive messages directed to another controller, PLCI or NCCI" );
		case 0x1104:
		   return _( "Queue is empty" );
		case 0x1105:
		   return _( "Queue overflow, a message was lost !! This indicates a configuration error. The only recovery from this error is to perform a CAPI_RELEASE" );
		case 0x1106:
		   return _( "Unknown notification parameter" );
		case 0x1107:
		   return _( "The Message could not be accepted because of an internal busy condition" );
		case 0x1108:
		   return _( "OS Resource error (no memory ?)" );
		case 0x1109:
		   return _( "CAPI not installed" );
		case 0x110A:
		   return _( "Controller does not support external equipment" );
		case 0x110B:
		   return _( "Controller does only support external equipment" );

		/*-- error information concerning resource / coding problems -----*/
		case 0x2001:
		   return _( "Message not supported in current state" );
		case 0x2002:
		   return _( "Illegal Controller / PLCI / NCCI" );
		case 0x2003:
		   return _( "Out of PLCI" );
		case 0x2004:
		   return _( "Out of NCCI" );
		case 0x2005:
		   return _( "Out of LISTEN" );
		case 0x2006:
		   return _( "Out of FAX resources (protocol T.30)" );
		case 0x2007:
		   return _( "Illegal message parameter coding" );

		/*-- error information concerning requested services  -----*/
		case 0x3001:
		   return _( "B1 protocol not supported" );
		case 0x3002: 
		   return _( "B2 protocol not supported" );
		case 0x3003: 
		   return _( "B3 protocol not supported" );
		case 0x3004: 
		   return _( "B1 protocol parameter not supported" );
		case 0x3005: 
		   return _( "B2 protocol parameter not supported" );
		case 0x3006: 
		   return _( "B3 protocol parameter not supported" );
		case 0x3007: 
		   return _( "B protocol combination not supported" );
		case 0x3008: 
		   return _( "NCPI not supported" );
		case 0x3009: 
		   return _( "CIP Value unknown" );
		case 0x300A: 
		   return _( "Flags not supported (reserved bits)" );
		case 0x300B: 
		   return _( "Facility not supported" );
		case 0x300C: 
		   return _( "Data length not supported by current protocol" );
		case 0x300D: 
		   return _( "Reset procedure not supported by current protocol" );

		/*-- informations about the clearing of a physical connection -----*/
		case 0x3301: 
		   return _( "Protocol error layer 1 (broken line or B-channel removed by signalling protocol)" );
		case 0x3302: 
		   return _( "Protocol error layer 2" );
		case 0x3303: 
		   return _( "Protocol error layer 3" );
		case 0x3304: 
		   return _( "Another application got that call" );

		/*-- T.30 specific reasons -----*/
		case 0x3311: 
		   return _( "Connecting not successful (remote station is no FAX G3 machine)" );
		case 0x3312: 
		   return _( "Connecting not successful (training error)" );
		case 0x3313: 
		   return _( "Disconnected before transfer (remote station does not support transfer mode, e.g. resolution)" );
		case 0x3314: 
		   return _( "Disconnected during transfer (remote abort)" );
		case 0x3315: 
		   return _( "Disconnected during transfer (remote procedure error, e.g. unsuccessful repetition of T.30 commands)" );
		case 0x3316: 
		   return _( "Disconnected during transfer (local tx data underrun)" );
		case 0x3317: 
		   return _( "Disconnected during transfer (local rx data overflow)" );
		case 0x3318: 
		   return _( "Disconnected during transfer (local abort)" );
		case 0x3319: 
		   return _( "Illegal parameter coding (e.g. SFF coding error)" );

		/*-- disconnect causes from the network according to ETS 300 102-1/Q.931 -----*/
		case 0x3481: return _( "Unallocated (unassigned) number" );
		case 0x3482: return _( "No route to specified transit network" );
		case 0x3483: return _( "No route to destination" );
		case 0x3486: return _( "Channel unacceptable" );
		case 0x3487: 
		   return _( "Call awarded and being delivered in an established channel" );
		case 0x3490: return _( "Normal call clearing" );
		case 0x3491: return _( "User busy" );
		case 0x3492: return _( "No user responding" );
		case 0x3493: return _( "No answer from user (user alerted)" );
		case 0x3495: return _( "Call rejected" );
		case 0x3496: return _( "Number changed" );
		case 0x349A: return _( "Non-selected user clearing" );
		case 0x349B: return _( "Destination out of order" );
		case 0x349C: return _( "Invalid number format" );
		case 0x349D: return _( "Facility rejected" );
		case 0x349E: return _( "Response to STATUS ENQUIRY" );
		case 0x349F: return _( "Normal, unspecified" );
		case 0x34A2: return _( "No circuit / channel available" );
		case 0x34A6: return _( "Network out of order" );
		case 0x34A9: return _( "Temporary failure" );
		case 0x34AA: return _( "Switching equipment congestion" );
		case 0x34AB: return _( "Access information discarded" );
		case 0x34AC: return _( "Requested circuit / channel not available" );
		case 0x34AF: return _( "Resources unavailable, unspecified" );
		case 0x34B1: return _( "Quality of service unavailable" );
		case 0x34B2: return _( "Requested facility not subscribed" );
		case 0x34B9: return _( "Bearer capability not authorized" );
		case 0x34BA: return _( "Bearer capability not presently available" );
		case 0x34BF: return _( "Service or option not available, unspecified" );
		case 0x34C1: return _( "Bearer capability not implemented" );
		case 0x34C2: return _( "Channel type not implemented" );
		case 0x34C5: return _( "Requested facility not implemented" );
		case 0x34C6: return _( "Only restricted digital information bearer capability is available" );
		case 0x34CF: return _( "Service or option not implemented, unspecified" );
		case 0x34D1: return _( "Invalid call reference value" );
		case 0x34D2: return _( "Identified channel does not exist" );
		case 0x34D3: return _( "A suspended call exists, but this call identity does not" );
		case 0x34D4: return _( "Call identity in use" );
		case 0x34D5: return _( "No call suspended" );
		case 0x34D6: return _( "Call having the requested call identity has been cleared" );
		case 0x34D8: return _( "Incompatible destination" );
		case 0x34DB: return _( "Invalid transit network selection" );
		case 0x34DF: return _( "Invalid message, unspecified" );
		case 0x34E0: return _( "Mandatory information element is missing" );
		case 0x34E1: return _( "Message type non-existent or not implemented" );
		case 0x34E2: return _( "Message not compatible with call state or message type non-existent or not implemented" );
		case 0x34E3: return _( "Information element non-existent or not implemented" );
		case 0x34E4: return _( "Invalid information element contents" );
		case 0x34E5: return _( "Message not compatible with call state" );
		case 0x34E6: return _( "Recovery on timer expiry" );
		case 0x34EF: return _( "Protocol error, unspecified" );
		case 0x34FF: return _( "Interworking, unspecified" );

		case 0x3500: return _( "Normal end of connection" );
		case 0x3501: return _( "Carrier lost" );
		case 0x3502: return _( "Error in negotation, i.e. no modem with error correction at the other end" );
		case 0x3503: return _( "No answer to protocol request" );
		case 0x3504: return _( "Remote modem only works in synchronous mode" );
		case 0x3505: return _( "Framing fails" );
		case 0x3506: return _( "Protocol negotiation fails" );
		case 0x3507: return _( "Other modem sends wrong protocol request" );
		case 0x3508: return _( "Sync information (data or flags) missing" );
		case 0x3509: return _( "Normal end of connection from the other modem" );
		case 0x350A: return _( "No answer from other modem" );
		case 0x350B: return _( "Protocol error" );
		case 0x350C: return _( "Error in compression" );
		case 0x350D: return _( "No conenct (timeout or wrong modulation)" );
		case 0x350E: return _( "No protocol fall-back allowed" );
		case 0x350F: return _( "No modem or fax at requested number" );
		case 0x3510: return _( "Handshake error" );

		default: return _( "No additional information" );
    }
}

/**
 * \brief Start capifax thread pool
 * \param pUserData user data pointer
 * \return NULL
 */
static gpointer capifaxThread( gpointer pUserData ) {
	capifax_loop();

	return NULL;
}

/**
 * \brief Progress function
 * \param nConnection connection id
 * \param psStatus capifax status pointer
 */
static void ProgressCapiFax( int nConnection, capifax_status_t *psStatus ) {
	float fTmp = 0;
	gchar anTmp[ 5 ];

	if ( psSendWindow == NULL ) {
		return;
	}

	gdk_threads_enter();
	if ( bSending == true ) {
		//printf( "[%s]: %ld %d\n", __FUNCTION__, psStatus -> bytes_sent, nCurrentSize );
		/*if ( psStatus -> bytes_total != 0 ) {
			fTmp = ( float ) psStatus -> bytes_sent / ( float ) psStatus -> bytes_total;
			gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( psProgressBar ), fTmp );
			fTmp *= 100.0f;
			snprintf( anTmp, sizeof( anTmp ), "%d%%", ( int )( fTmp ) );
			gtk_progress_bar_set_text( GTK_PROGRESS_BAR( psProgressBar ), anTmp );
			Debug( KERN_DEBUG, "1. %d %d\n", psStatus -> bytes_sent, psStatus -> bytes_total );
		} else*/ {
			fTmp = ( float ) psStatus -> bytes_sent / ( float ) nCurrentSize;
			gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( psProgressBar ), fTmp );
			fTmp *= 100.0f;
			snprintf( anTmp, sizeof( anTmp ), "%d%%", ( int )( fTmp ) );
			gtk_progress_bar_set_text( GTK_PROGRESS_BAR( psProgressBar ), anTmp );
			//Debug( KERN_DEBUG, "2. %d %d\n", psStatus -> bytes_sent, nCurrentSize );
		}
		//Debug( KERN_DEBUG, "fTmp: %f\n", fTmp );
	} else {
		gtk_progress_bar_pulse( GTK_PROGRESS_BAR( psProgressBar ) );
	}
	gdk_threads_leave();
}

/**
 * \brief Set fax button according to window type
 * \param nType type of window
 */
static void faxSetButtons( gint nType ) {
	if ( psCancelButton != NULL ) {
		gtk_widget_destroy( psCancelButton );
	}
	if ( psOkButton != NULL ) {
		gtk_widget_destroy( psOkButton );
	}

	if ( nType == 0 ) {
		/* Add cancel button and enable it */
		psCancelButton = gtk_dialog_add_button( GTK_DIALOG( psSendWindow ), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL );
		gtk_widget_set_sensitive( psCancelButton, TRUE );

		/* Add ok button and disable it */
		psOkButton = gtk_dialog_add_button( GTK_DIALOG( psSendWindow ), GTK_STOCK_OK, GTK_RESPONSE_OK );
		gtk_widget_set_sensitive( psOkButton, FALSE );
	} else {
		/* Add redo button and enable it */
		psCancelButton = gtk_dialog_add_button( GTK_DIALOG( psSendWindow ), GTK_STOCK_REDO, 1 );
		gtk_widget_set_sensitive( psCancelButton, TRUE );

		/* Add ok button and disable it */
		psOkButton = gtk_dialog_add_button( GTK_DIALOG( psSendWindow ), GTK_STOCK_OK, GTK_RESPONSE_OK );
		gtk_widget_set_sensitive( psOkButton, TRUE );
	}

	gtk_widget_show( psCancelButton );
	gtk_widget_show( psOkButton );
}

/**
 * \brief Status function
 * \param nConnection connection id
 * \param psStatus capifax status pointer
 */
static void StatusCapiFax( int nConnection, capifax_status_t *psStatus ) {
	char anBuffer[ 256 ];

	if ( psSendWindow == NULL ) {
		return;
	}

	gdk_threads_enter();
	switch ( psStatus -> phase ) {
		case IDLE:
			break;
		case CONNECT:
			if ( bSending == true ) {
				Debug( KERN_DEBUG, "Outgoing fax\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Outgoing fax</b>" ) );
			} else {
				Debug( KERN_DEBUG, "Incoming fax\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Incoming fax</b>" ) );
			}
			break;
		case PHASE_B:
			if ( bSending == true ) {
				Debug( KERN_DEBUG, "Sending fax (START)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Sending fax (START)</b>" ) );
			} else {
				Debug( KERN_DEBUG, "Receiving fax (START)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Receiving fax (START)</b>" ) );
			}
			break;
		case PHASE_D:
			if ( bSending == true ) {
				Debug( KERN_DEBUG, "Sending fax (PAGE)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Sending fax (PAGE)</b>" ) );
			} else {
				Debug( KERN_DEBUG, "Receiving fax (PAGE)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Receiving fax (PAGE)</b>" ) );
			}
			break;
		case PHASE_E:
			if ( bSending == true ) {
				Debug( KERN_DEBUG, "Sending fax (END)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Sending fax (END)</b>" ) );
			} else {
				Debug( KERN_DEBUG, "Receiving fax (END)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Receiving fax (END)</b>" ) );
			}
			break;
		case FAX_OK:
			if ( bSending == true ) {
				Debug( KERN_DEBUG, "Sending fax (SUCCESSFUL)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Sending fax (SUCCESSFUL)</b>" ) );
			} else {
				Debug( KERN_DEBUG, "Receiving fax (SUCCESSFUL)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Receiving fax (SUCCESSFUL)</b>" ) );
			}
			gtk_widget_set_sensitive( psOkButton, TRUE );
			gtk_widget_set_sensitive( psCancelButton, FALSE );
			gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( psProgressBar ), 1.0 );
			gtk_progress_bar_set_text( GTK_PROGRESS_BAR( psProgressBar ), "100%%" );
			if ( bSending == FALSE ) {
				g_source_remove( nTimer );
			}
			CreateFaxReport( pnCurrentFile, pnCurrentReportDir, psStatus -> trg_no, psStatus -> src_no, "SUCCESS", capiInfo2String( psStatus -> capi_code ), psStatus -> page_total, psStatus -> bitrate );
			break;
		case FAX_FAILED:
			if ( bSending == true ) {
				Debug( KERN_DEBUG, "Sending fax (FAILED)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Sending fax (FAILED)</b>" ) );
				faxSetButtons( 1 );
			} else {
				Debug( KERN_DEBUG, "Receiving fax (FAILED)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Receiving fax (FAILED)</b>" ) );
				g_source_remove( nTimer );
			}
			gtk_label_set_text( GTK_LABEL( psErrorLabel ), capiInfo2String( psStatus -> capi_code ) );
			gtk_widget_set_sensitive( psOkButton, TRUE );
			gtk_widget_set_sensitive( psCancelButton, FALSE );
			gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( psProgressBar ), 1.0 );
			CreateFaxReport( pnCurrentFile, pnCurrentReportDir, psStatus -> trg_no, psStatus -> src_no, "FAILED", capiInfo2String( psStatus -> capi_code ), psStatus -> page_total, psStatus -> bitrate );
			break;
		case FAX_REJECTED:
			if ( bSending == true ) {
				Debug( KERN_DEBUG, "Sending fax (REJECTED)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Sending fax (REJECTED)</b>" ) );
			} else {
				Debug( KERN_DEBUG, "Receiving fax (REJECTED)\n" );
				gtk_label_set_markup( GTK_LABEL( psStatusLabel ), _( "<b>Receiving fax (REJECTED)</b>" ) );
			}
			gtk_label_set_text( GTK_LABEL( psErrorLabel ), capiInfo2String( psStatus -> capi_code ) );
			gtk_widget_set_sensitive( psOkButton, TRUE );
			gtk_widget_set_sensitive( psCancelButton, FALSE );
			faxSetButtons( 1 );
			gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( psProgressBar ), 1.0 );
			if ( bSending == FALSE ) {
				g_source_remove( nTimer );
			}
			CreateFaxReport( pnCurrentFile, pnCurrentReportDir, psStatus -> trg_no, psStatus -> src_no, "REJECTED", capiInfo2String( psStatus -> capi_code ), psStatus -> page_total, psStatus -> bitrate );
			break;
	}

	Debug( KERN_DEBUG, "Phase     : %d\n", psStatus -> phase );
	Debug( KERN_DEBUG, "Connection: %d\n", nConnection );
	if ( psStatus -> phase != 1 ) {
		Debug( KERN_DEBUG, "Bitrate   : %d\n", psStatus -> bitrate );
		snprintf( anBuffer, sizeof( anBuffer ), "%d", psStatus -> bitrate );
		gtk_label_set_text( GTK_LABEL( psBitrateNumber ), anBuffer );
		Debug( KERN_DEBUG, "Encoding  : %s\n", t4_encoding_to_str( psStatus -> encoding ) );
		gtk_label_set_text( GTK_LABEL( psEncodingNumber ), t4_encoding_to_str( psStatus -> encoding ) );
		Debug( KERN_DEBUG, "Bad rows  : %d\n", psStatus -> bad_rows );
		snprintf( anBuffer, sizeof( anBuffer ), "%d", psStatus -> bad_rows );
		gtk_label_set_text( GTK_LABEL( psBadRowsNumber ), anBuffer );
		Debug( KERN_DEBUG, "Page      : %d/%d\n", psStatus -> page_current, psStatus -> page_total );
		snprintf( anBuffer, sizeof( anBuffer ), "%d/%d", psStatus -> page_current, psStatus -> page_total );
		gtk_label_set_text( GTK_LABEL( psPageNumber ), anBuffer );
	} else {
		Debug( KERN_DEBUG, "Src-No    : %s\n", psStatus -> src_no );
		Debug( KERN_DEBUG, "Dst-No    : %s\n", psStatus -> trg_no );
		snprintf( anBuffer, sizeof( anBuffer ), "%s", psStatus -> tiff_file );
		gtk_label_set_text( GTK_LABEL( psErrorLabel ), anBuffer );
	}
	gdk_threads_leave();
}

/**
 * \brief Log capifax messages
 * \param pnMessage
 */
static void LogCapiFax( char *pnMessage ) {
	Debug( KERN_DEBUG, "%s\n", pnMessage );
}

/**
 * \brief Move progress bar
 * \param pData user data pointer
 * \return TRUE (do not delete timer)
 */
static gboolean updateProgressBar( gpointer pData ) {
	gtk_progress_bar_pulse( GTK_PROGRESS_BAR( pData ) );

	return TRUE;
}

/**
 * \brief Send window response callback
 * \param psDialog dialog window pointer
 * \param nResponse response id
 * \param pUserData user data pointer
 */
static void sendWindowResponse( GtkDialog *psDialog, gint nResponse, gpointer pUserData ) {
	gint nConnection = GPOINTER_TO_INT( pUserData );

	switch ( nResponse ) {
		case 1:
			nConnection = capifax_send( pnCurrentFile, nCurrentBitrate, nCurrentEcm, nCurrentController, pnCurrentSenderMsn, pnCurrentNumber, pnCurrentSenderNumber, pnCurrentSenderName, nCurrentCallAnonymous );
			if ( bSending == FALSE ) {
				nTimer = g_timeout_add( 100, updateProgressBar, psProgressBar );
			}
			faxSetButtons( 0 );
			return;
		case GTK_RESPONSE_OK:
			break;
		case GTK_RESPONSE_CANCEL:
		case GTK_RESPONSE_DELETE_EVENT:
			capifax_terminate( nConnection );
			g_source_remove( nTimer );
			break;
	}

	gtk_widget_destroy( GTK_WIDGET( psDialog ) );
	psSendWindow = NULL;

	if ( pnCurrentSenderMsn != NULL ) {
		g_free( pnCurrentSenderMsn );
		pnCurrentSenderMsn = NULL;
	}
	if ( pnCurrentNumber != NULL ) {
		g_free( pnCurrentNumber );
		pnCurrentNumber = NULL;
	}
	if ( pnCurrentSenderNumber != NULL ) {
		g_free( pnCurrentSenderNumber );
		pnCurrentSenderNumber = NULL;
	}
	if ( pnCurrentSenderName != NULL ) {
		g_free( pnCurrentSenderName );
		pnCurrentSenderName = NULL;
	}

	if ( pnCurrentReportDir != NULL ) {
		g_free( pnCurrentReportDir );
		pnCurrentReportDir = NULL;
	}

	if ( bSending == TRUE ) {
		unlink( pnCurrentFile );
	}

	if ( pnCurrentFile != NULL ) {
		g_free( pnCurrentFile );
		pnCurrentFile = NULL;
	}
}

/**
 * \brief Creates fax status windows, used by sending/receiving
 * \param pnSenderNumber sender number
 * \param pnNumber target number
 * \param nConnection connection id
 */
void createFaxStatusWindow( const char *pnSenderNumber, char *pnNumber, int nConnection ) {
	GtkWidget *psTable;
	GtkWidget *psExpander;
	GtkWidget *psExpanderTable;
	GtkWidget *psSourceLabel, *psSourceNumber;
	GtkWidget *psDestinationLabel, *psDestinationNumber;
	GtkWidget *psBitrateLabel;
	GtkWidget *psEncodingLabel;
	GtkWidget *psBadRowsLabel;
	GtkWidget *psPageLabel;

	psSendWindow = gtk_dialog_new_with_buttons( _( "Fax Status" ), NULL, GTK_DIALOG_DESTROY_WITH_PARENT, NULL );

	psCancelButton = NULL;
	psOkButton = NULL;
	faxSetButtons( 0 );

	/* create main table */
	psTable = gtk_table_new( 4, 2, false );

	/* Create fax status label */
	psStatusLabel = gtk_label_new( "" );
	gtk_misc_set_alignment( GTK_MISC( psStatusLabel ), 0.5, 0.5 );
	gtk_table_attach( GTK_TABLE( psTable ), psStatusLabel, 0, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0 );

	psErrorLabel = gtk_label_new( "" );
	gtk_misc_set_alignment( GTK_MISC( psErrorLabel ), 0.5, 0.5 );
	gtk_table_attach( GTK_TABLE( psTable ), psErrorLabel, 0, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0 );

	/* Create Progressbar */
	psProgressBar = gtk_progress_bar_new();
	if ( bSending == false ) {
		gtk_progress_bar_pulse( GTK_PROGRESS_BAR( psProgressBar ) );
	}
	gtk_table_attach( GTK_TABLE( psTable ), psProgressBar, 0, 2, 2, 3, GTK_FILL | GTK_EXPAND, 0, 0, 0 );

	/* Create expander */
	psExpander = gtk_expander_new( _( "More Information" ) );
	gtk_table_attach( GTK_TABLE( psTable ), psExpander, 0, 2, 3, 4, GTK_FILL, 0, 0, 0 );

	psExpanderTable = gtk_table_new( 4, 2, false );
	gtk_container_add( GTK_CONTAINER( psExpander ), GTK_WIDGET( psExpanderTable ) );

	/* Source label and number */
	psSourceLabel = gtk_label_new( _( "Sender-Number:" ) );
	gtk_misc_set_alignment( GTK_MISC( psSourceLabel ), 0, 0.5 );
	gtk_misc_set_padding( GTK_MISC( psSourceLabel ), 25, 0 );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psSourceLabel, 0, 1, 0, 1, GTK_FILL, 0, 0, 0 );

	psSourceNumber = gtk_label_new( pnSenderNumber );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psSourceNumber, 1, 2, 0, 1, GTK_FILL, 0, 0, 0 );

	/* Destination label and number */
	psDestinationLabel = gtk_label_new( _( "Destination-Number:" ) );
	gtk_misc_set_alignment( GTK_MISC( psDestinationLabel ), 0, 0.5 );
	gtk_misc_set_padding( GTK_MISC( psDestinationLabel ), 25, 0 );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psDestinationLabel, 0, 1, 1, 2, GTK_FILL, 0, 0, 0 );

	psDestinationNumber = gtk_label_new( pnNumber );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psDestinationNumber, 1, 2, 1, 2, GTK_FILL, 0, 0, 0 );

	/* Bitrate label and number */
	psBitrateLabel = gtk_label_new( _( "Bitrate:" ) );
	gtk_misc_set_alignment( GTK_MISC( psBitrateLabel ), 0, 0.5 );
	gtk_misc_set_padding( GTK_MISC( psBitrateLabel ), 25, 0 );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psBitrateLabel, 0, 1, 2, 3, GTK_FILL, 0, 0, 0 );

	psBitrateNumber = gtk_label_new( "-----" );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psBitrateNumber, 1, 2, 2, 3, GTK_FILL, 0, 0, 0 );

	/* Encoding label and number */
	psEncodingLabel = gtk_label_new( _( "Encoding:" ) );
	gtk_misc_set_alignment( GTK_MISC( psEncodingLabel ), 0, 0.5 );
	gtk_misc_set_padding( GTK_MISC( psEncodingLabel ), 25, 0 );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psEncodingLabel, 0, 1, 3, 4, GTK_FILL, 0, 0, 0 );

	psEncodingNumber = gtk_label_new( "-" );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psEncodingNumber, 1, 2, 3, 4, GTK_FILL, 0, 0, 0 );

	/* BadRows label and number */
	psBadRowsLabel = gtk_label_new( _( "Bad Rows:" ) );
	gtk_misc_set_alignment( GTK_MISC( psBadRowsLabel ), 0, 0.5 );
	gtk_misc_set_padding( GTK_MISC( psBadRowsLabel ), 25, 0 );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psBadRowsLabel, 0, 1, 4, 5, GTK_FILL, 0, 0, 0 );

	psBadRowsNumber = gtk_label_new( "-" );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psBadRowsNumber, 1, 2, 4, 5, GTK_FILL, 0, 0, 0 );

	/* Page label and number */
	psPageLabel = gtk_label_new( _( "Page:" ) );
	gtk_misc_set_alignment( GTK_MISC( psPageLabel ), 0, 0.5 );
	gtk_misc_set_padding( GTK_MISC( psPageLabel ), 25, 0 );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psPageLabel, 0, 1, 5, 6, GTK_FILL, 0, 0, 0 );

	psPageNumber = gtk_label_new( "-" );
	gtk_table_attach( GTK_TABLE( psExpanderTable ), psPageNumber, 1, 2, 5, 6, GTK_FILL, 0, 0, 0 );

	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( psSendWindow ) -> vbox ), psTable, false, false, 10 );

	g_signal_connect( G_OBJECT( psSendWindow ), "response", G_CALLBACK( sendWindowResponse ), GINT_TO_POINTER( nConnection ) );

	if ( bSending == FALSE ) {
		nTimer = g_timeout_add( 100, updateProgressBar, psProgressBar );
	}

	gtk_window_set_default_size( GTK_WINDOW( psSendWindow ), 380, -1 );
	gtk_widget_show_all( psSendWindow );
}

/**
 * \brief Send fax through capi
 * \param pnFile filename to fax
 * \param nBitrate bitrate
 * \param nEcm error correction mode
 * \param nController controller id
 * \param pnSenderMsn sender msn
 * \param pnNumber number
 * \param pnSenderNumber sender number
 * \param pnSenderName sender name
 * \param nCallAnonymous call anonymous flag
 * \param pnReportDir directory where to store report
 * \return error code
 */
static int SendCapiFax( const char *pnFile, int nBitrate, int nEcm, int nController, const char *pnSenderMsn, const char *pnNumber, const char *pnSenderNumber, const char *pnSenderName, int nCallAnonymous, char *pnReportDir ) {
	int nConnection;
	int nHandle = -1;

	if ( nCapiFaxInit != 0 ) {
		ShowError( FALSE, _( "Cannot send fax" ), _( "capifax not initialized!" ) );
		unlink( pnFile );
		return -1;
	}

	pnCurrentFile = g_strdup( pnFile );
	nHandle = open( pnCurrentFile, O_RDONLY );
	if ( nHandle != -1 ) {
		nCurrentSize = lseek( nHandle, 0, SEEK_END );
		close( nHandle );
	} else {
		if ( pnCurrentFile != NULL ) {
			g_free( pnCurrentFile );
			pnCurrentFile = NULL;
		}
		unlink( pnFile );
		return -2;
	}

	bSending = true;
	if ( pnReportDir != NULL ) {
		pnCurrentReportDir = g_strdup( pnReportDir );
	} else {
		pnCurrentReportDir = NULL;
	}

	nCurrentBitrate = nBitrate;
	nCurrentEcm = nEcm;
	nCurrentController = nController + 1;
	pnCurrentSenderMsn = g_strdup( pnSenderMsn );
	pnCurrentNumber = g_strdup( pnNumber );
	pnCurrentSenderNumber = g_strdup( pnSenderNumber );
	pnCurrentSenderName = g_strdup( pnSenderName );
	nCurrentCallAnonymous = nCallAnonymous;

	nConnection = capifax_send( pnFile, nBitrate, nEcm, nController + 1, pnSenderMsn, pnNumber, pnSenderNumber, pnSenderName, nCallAnonymous );

	createFaxStatusWindow( pnSenderNumber, ( char * ) pnNumber, nConnection );

	return 0;
}

/**
 * \brief Receive fax callback
 * \param nConnection capi connection id
 * \param pnSenderNumber sender number
 * \param pnNumber target number
 * \param ppnFile fax file name
 * \return handled flag
 */
static int ReceiveCallback( int nConnection, char *pnSenderNumber, char *pnNumber, char **ppnFile ) {
	time_t tim = time( NULL );
	struct tm *psNow = localtime( &tim );
	gchar *pnTmp;

	Debug( KERN_DEBUG, "check1 - do we accept incoming fax?\n" );
	if ( faxGetAccept( getActiveProfile() ) == FALSE ) {
		Debug( KERN_DEBUG, "-> no\n" );
		return -1;
	}

	Debug( KERN_DEBUG, "check2 - valid local number?\n" );

	pnTmp = strchr( pnSenderNumber, '#' );
	if ( pnTmp != NULL ) {
		if ( strcmp( pnTmp + 1, faxGetSenderMsn( getActiveProfile() ) ) ) {
			Debug( KERN_DEBUG, "-> not the fax number we want (%s<->%s)\n", pnTmp + 1, faxGetSenderMsn( getActiveProfile() ) ); 
			return -2;
		}
	} else {
		if ( strcmp( pnSenderNumber, faxGetSenderMsn( getActiveProfile() ) ) && strcmp( pnSenderNumber, "unknown" ) ) {
			Debug( KERN_DEBUG, "-> not the fax number we want (%s<->%s)\n", pnSenderNumber, faxGetSenderMsn( getActiveProfile() ) );
			return -2;
		}
	}

	Debug( KERN_DEBUG, "check3 - transfer already active?\n" );
	if ( pnCurrentFile != NULL ) {
		Debug( KERN_DEBUG, "-> we are already active, abort\n" );
		return -3;
	}

	*ppnFile = g_strdup_printf( "%s/ffgtk_%s_%s-%02d_%02d_%d_%02d_%02d_%02d.tiff", faxGetReportDir( getActiveProfile() ),
		pnSenderNumber, pnNumber, psNow -> tm_mday, psNow -> tm_mon + 1, psNow -> tm_year + 1900, psNow -> tm_hour, psNow -> tm_min, psNow -> tm_sec );

	pnCurrentFile = *ppnFile;
	Debug( KERN_DEBUG, "Saving fax to '%s'\n", pnCurrentFile );

	bSending = false;
	pnCurrentReportDir = NULL;
	Debug( KERN_DEBUG, "create fax window\n" );
	gdk_threads_enter();
	createFaxStatusWindow( pnNumber, pnSenderNumber, nConnection );
	gdk_threads_leave();
	Debug( KERN_DEBUG, "done\n" );

	return 0;
}

/**
 * \brief Initialize capifax library
 * \param nController controller id
 * \param nLogLevel log level
 * \return error code (see capifax_init)
 */
static int InitCapiFax( int nController, int nLogLevel ) {
	nCapiFaxInit = capifax_init( -1, nLogLevel, StatusCapiFax, ProgressCapiFax, LogCapiFax, ReceiveCallback );

	if ( nCapiFaxInit == 0 ) {
		g_thread_create( capifaxThread, NULL, false, NULL );
	} else {
		Debug( KERN_WARNING, "Could not init capifax!!\n" );
	}

	return nCapiFaxInit;
}

/** fax defintions */
static struct sFax sCapiFax = {
	InitCapiFax,
	NULL,
	SendCapiFax,
	capifax_close
};

/**
 * \brief Close plugin
 * \return error code
 */
static int ClosePlugin( void ) {
	return capifax_close();
}

MODULE_INIT( PLUGIN_TYPE_FAX, _( "CapiFax" ), &sCapiFax, ClosePlugin, NULL );
