/*****************************************************************
 * Fritz Fun                                                     *
 * Created by Jan-Michael Brummer                                *
 * All parts are distributed under the terms of GPL. See COPYING *
 *****************************************************************/

/**
 * \file voxplay.c
 * \brief Play voice box speex files from FRITZ!Box
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <speex/speex_callbacks.h>

#ifndef OLD
#include <ao/ao.h>
#include <string.h>
#else
#include <sys/soundcard.h>
#endif

#define MAX_FRAME_SIZE 2000

#ifdef OLD
FILE *out_file_open( int rate ) {
	FILE *fout=NULL;
	int audio_fd, format, stereo;

	audio_fd = open("/dev/dsp", O_WRONLY);
	if ( audio_fd < 0 ) {
		perror("Cannot open /dev/dsp");
		return NULL;
	}

	format = AFMT_S16_NE;
	if ( ioctl( audio_fd, SNDCTL_DSP_SETFMT, &format ) == -1 ) {
		perror("SNDCTL_DSP_SETFMT");
		close(audio_fd);
		return NULL;
	}

	stereo = 0;
	if ( ioctl( audio_fd, SNDCTL_DSP_STEREO, &stereo ) == -1 ) {
		perror("SNDCTL_DSP_STEREO");
		close(audio_fd);
		return NULL;
	}
	if ( stereo != 0 ) {
		fprintf (stderr, "Cannot set mono mode, will decode in stereo\n");
	}

	if ( ioctl( audio_fd, SNDCTL_DSP_SPEED, &rate ) == -1 ) {
		perror("SNDCTL_DSP_SPEED");
		close(audio_fd);
		return NULL;
	}

	fout = fdopen( audio_fd, "w" );

	return fout;
}
#else
ao_device *openAudio(void) {
	ao_device *device;
	ao_sample_format format;
    int default_driver;

	ao_initialize();

	default_driver = ao_default_driver_id();

	memset(&format, 0, sizeof(format));
	format.bits = 16;
	format.channels = 1;
	format.rate = 8000;
	format.byte_format = AO_FMT_LITTLE;

	device = ao_open_live(default_driver, &format, NULL);
	return device;
}
#endif

int main( int argc, char **argv ) {
	char *inFile;
	FILE *fin = NULL;
	short output[MAX_FRAME_SIZE];
	spx_int32_t frame_size = 0;
	void *st = NULL;
	SpeexBits bits;
	spx_int32_t rate=0;
	unsigned char bytes = 0;
	char packetbuffer[38];
	int j, nb_read;
	const SpeexMode *mode;

	inFile = argv[ 1 ];
	fin = fopen(inFile, "rb");
	if ( !fin ) {
		return -1;
	}

	speex_bits_init( &bits );

	rate = 8000;

	mode = speex_lib_get_mode (0);

	st = speex_decoder_init(mode);
	if (!st) {
		fprintf (stderr, "Decoder initialization failed.\n");
		return -1;
	}
	speex_decoder_ctl(st, SPEEX_GET_FRAME_SIZE, &frame_size);
	speex_decoder_ctl(st, SPEEX_SET_SAMPLING_RATE, &rate);

#ifdef OLD
	FILE *fout = NULL;

	fout = out_file_open(rate);
	if ( fout == NULL ) {
		fclose( fin );
	}
#else
	ao_device *device = openAudio();
	if (device == NULL) {
		fprintf(stderr, "Error opening device.\n");
		return 1;
	}
#endif

	while ( !feof( fin ) ) {
		nb_read = fread(&bytes, sizeof(char), 1, fin);

		if ( bytes != 0x26 ) {
			continue;
		}

		nb_read = fread(packetbuffer, sizeof(char), (int)bytes, fin );
		if ( nb_read != sizeof( char ) * bytes ) {
			break;
		}
		speex_bits_read_from(&bits, (char *)packetbuffer, bytes);

		for ( j = 0; j != 2; j++ ) {
			int ret;

			ret = speex_decode_int(st, &bits, output);

			if ( ret == -1 ) {
				break;
			} else if ( ret == -2 ) {
				fprintf (stderr, "Decoding error: corrupted stream?\n");
				break;
			}

			if ( speex_bits_remaining( &bits ) < 0 ) {
				fprintf (stderr, "Decoding overflow: corrupted stream?\n");
				break;
			}

		}

#ifdef OLD
		if ( fwrite(output, sizeof(short), frame_size, fout) != frame_size ) {
			fprintf( stderr, "Could not play frame!\n" );
			break;
		}
#else
		ao_play(device, (char*)output, frame_size * sizeof(short));
#endif
	}

	speex_decoder_destroy(st);
	speex_bits_destroy(&bits);

	fclose(fin);

#ifdef OLD
	fclose(fout);
#else
	ao_close(device);
	ao_shutdown();
#endif

	return 0;
}
